@@ -46,12 +46,12 @@ static const char laurent[]     = I18N_NOOP("Porting to KDE 4 when no one else w
 int main(int argc, char *argv[])
 {
     KAboutData aboutData("juk", 0, ki18n("JuK"),
-                         "3.4.1", ki18n(description), KAboutData::License_GPL,
-                         ki18n("© 2002 - 2009, Scott Wheeler"), KLocalizedString(),
+                         "3.5", ki18n(description), KAboutData::License_GPL,
+                         ki18n("© 2002 - 2010, Scott Wheeler"), KLocalizedString(),
                          "http://developer.kde.org/~wheeler/juk.html");
 
     aboutData.addAuthor(ki18n("Scott Wheeler"), ki18n(scott), "wheeler@kde.org");
-    aboutData.addAuthor(ki18n("Michael Pyne"), ki18n(michael), "michael.pyne@kdemail.net");
+    aboutData.addAuthor(ki18n("Michael Pyne"), ki18n(michael), "mpyne@kde.org");
     aboutData.addCredit(ki18n("Daniel Molkentin"), ki18n(daniel), "molkentin@kde.org");
     aboutData.addCredit(ki18n("Tim Jansen"), ki18n(tim), "tim@tjansen.de");
     aboutData.addCredit(ki18n("Stefan Asserhäll"), ki18n(stefan), "stefan.asserhall@telia.com");
