@@ -65,8 +65,13 @@ public:
      * This constructor should generally only be used either by the cache
      * restoration methods or by subclasses that want to handle calls to
      * PlaylistCollection::setupPlaylist() differently.
+     *
+     * @param extraColumns is used to preallocate columns for subclasses that
+     * need them (since extra columns are assumed to start from 0). extraColumns
+     * should be equal to columnOffset() (we can't use columnOffset until the
+     * ctor has run).
      */
-    Playlist(PlaylistCollection *collection, bool delaySetup);
+    Playlist(PlaylistCollection *collection, bool delaySetup, int extraColumns = 0);
 
     virtual ~Playlist();
 
@@ -551,6 +556,13 @@ private:
 
     void updatePlaying() const;
 
+    /**
+     * This function should be called when item is deleted to ensure that any
+     * internal bookkeeping is performed.  It is automatically called by
+     * PlaylistItem::~PlaylistItem and by clearItem() and clearItems().
+     */
+    void updateDeletedItem(PlaylistItem *item);
+
     /**
      * Used as a helper to implement template<> createItem().  This grabs the
      * CollectionListItem for file if it exists, otherwise it creates a new one and
@@ -645,6 +657,8 @@ private slots:
     void slotPlayCurrent();
 
 private:
+    friend class PlaylistItem;
+
     PlaylistCollection *m_collection;
 
     StringHash m_members;
